#!/usr/bin/env node
'use strict';

var program = require('commander');
var Nedb = require('nedb');
var pjson = require('pjson');
var updateNotifier = require('update-notifier');
var axios = require('axios');
var prompts = require('prompts');
var open = require('open');
var ora = require('ora');
var fs = require('fs');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var program__default = /*#__PURE__*/_interopDefaultLegacy(program);
var Nedb__default = /*#__PURE__*/_interopDefaultLegacy(Nedb);
var pjson__default = /*#__PURE__*/_interopDefaultLegacy(pjson);
var updateNotifier__default = /*#__PURE__*/_interopDefaultLegacy(updateNotifier);
var axios__default = /*#__PURE__*/_interopDefaultLegacy(axios);
var prompts__default = /*#__PURE__*/_interopDefaultLegacy(prompts);
var open__default = /*#__PURE__*/_interopDefaultLegacy(open);
var ora__default = /*#__PURE__*/_interopDefaultLegacy(ora);
var fs__default = /*#__PURE__*/_interopDefaultLegacy(fs);

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

function __awaiter(thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

function __generator(thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
}

var color = function (col) { return function (str) { return "\u001B[" + col + "m" + str + "\u001B[0m"; }; };
var colors = {
    black: color('30'),
    red: color('31'),
    green: color('32'),
    yellow: color('33'),
    blue: color('34'),
    magenta: color('35'),
    cyan: color('36'),
    white: color('37')
};

var setUser = function (db, accessToken, accessTokenSecret, userName, id) { return __awaiter(void 0, void 0, void 0, function () {
    var data, result;
    return __generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                data = {
                    type: 'user',
                    name: userName,
                    accessToken: accessToken,
                    accessTokenSecret: accessTokenSecret,
                    userid: id.toString(),
                    selected: true
                };
                return [4 /*yield*/, new Promise(function (resolve) {
                        db.insert(data, function (err) {
                            if (!err) {
                                resolve(true);
                                return;
                            }
                            resolve(false);
                        });
                    })];
            case 1:
                result = _a.sent();
                return [4 /*yield*/, db.loadDatabase()];
            case 2:
                _a.sent();
                return [2 /*return*/, result];
        }
    });
}); };
var deleteUser = function (db, id) { return __awaiter(void 0, void 0, void 0, function () {
    var result;
    return __generator(this, function (_a) {
        switch (_a.label) {
            case 0: return [4 /*yield*/, new Promise(function (resolve) {
                    db.remove({ _id: id }, { multi: false }, function (err) {
                        if (!err) {
                            resolve(true);
                            return;
                        }
                        resolve(false);
                    });
                })];
            case 1:
                result = _a.sent();
                return [4 /*yield*/, db.loadDatabase()];
            case 2:
                _a.sent();
                return [2 /*return*/, result];
        }
    });
}); };
var setSelectedUser = function (db, name) { return __awaiter(void 0, void 0, void 0, function () {
    var data;
    return __generator(this, function (_a) {
        switch (_a.label) {
            case 0: return [4 /*yield*/, new Promise(function (resolve) {
                    db.update({
                        name: name
                    }, {
                        $set: {
                            selected: true
                        }
                    }, {}, function (error) {
                        if (!error) {
                            resolve(true);
                            return;
                        }
                        resolve(false);
                    });
                })];
            case 1:
                data = _a.sent();
                return [4 /*yield*/, db.loadDatabase()];
            case 2:
                _a.sent();
                return [2 /*return*/, data];
        }
    });
}); };
var unSetSelecte = function (db) { return __awaiter(void 0, void 0, void 0, function () {
    var data;
    return __generator(this, function (_a) {
        switch (_a.label) {
            case 0: return [4 /*yield*/, new Promise(function (resolve) {
                    db.update({
                        selected: true
                    }, {
                        $set: {
                            selected: false
                        }
                    }, {
                        multi: true
                    }, function (error) {
                        if (!error) {
                            resolve(true);
                            return;
                        }
                        resolve(false);
                    });
                })];
            case 1:
                data = _a.sent();
                return [4 /*yield*/, db.loadDatabase()];
            case 2:
                _a.sent();
                return [2 /*return*/, data];
        }
    });
}); };
var getAllUser = function (db) {
    return new Promise(function (resolve) {
        db.find({}, function (err, result) {
            if (!err && result) {
                resolve(result);
            }
            else {
                resolve([]);
            }
        });
    });
};
var getRandomUser = function (db) {
    return new Promise(function (resolve) {
        db.findOne({}, function (err, result) {
            if (!err && result) {
                resolve(result);
            }
            else {
                resolve({});
            }
        });
    });
};
var checkUserName = function (db, name) {
    return new Promise(function (resolve) {
        db.findOne({ name: name }, function (err, result) {
            if (!err && result) {
                resolve(true);
            }
            else {
                resolve(false);
            }
        });
    });
};
var selectedUser = function (db) { return __awaiter(void 0, void 0, void 0, function () {
    var user;
    return __generator(this, function (_a) {
        switch (_a.label) {
            case 0: return [4 /*yield*/, new Promise(function (resolve) {
                    db.findOne({ selected: true }, function (err, result) {
                        if (!err && result) {
                            resolve(result);
                        }
                        else {
                            resolve({});
                        }
                    });
                })];
            case 1:
                user = _a.sent();
                db.loadDatabase();
                return [2 /*return*/, user];
        }
    });
}); };

var middlewareUrl = "https://twcl-middleware.herokuapp.com";
var callbackUrl = "https://api.twitter.com/oauth";

var inputTweet = function () { return __awaiter(void 0, void 0, void 0, function () {
    var onCancel, input;
    return __generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                onCancel = function () {
                    throw new Error('入力内容が確認できませんでした...もう一度最初から入力してください。');
                };
                return [4 /*yield*/, prompts__default['default']([
                        {
                            type: 'text',
                            name: 'input',
                            message: 'ツイート: ',
                            validate: function (value) { return (!value ? '何か入力してください' : true); }
                        }
                    ], { onCancel: onCancel })];
            case 1:
                input = (_a.sent()).input;
                return [2 /*return*/, { input: input }];
        }
    });
}); };
var postTweet = function (accessToken, accessTokenSecret, tweet) { return __awaiter(void 0, void 0, void 0, function () {
    var err_1;
    return __generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                _a.trys.push([0, 2, , 3]);
                return [4 /*yield*/, axios__default['default'].post(middlewareUrl + "/postTweet", {
                        access_token: accessToken,
                        access_token_secret: accessTokenSecret,
                        tweet: tweet
                    })];
            case 1:
                _a.sent();
                return [2 /*return*/, true];
            case 2:
                err_1 = _a.sent();
                throw new Error(err_1.message);
            case 3: return [2 /*return*/];
        }
    });
}); };
var Tweet = function (user, tweet) { return __awaiter(void 0, void 0, void 0, function () {
    var accessToken, accessTokenSecret, input, err_2;
    return __generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                _a.trys.push([0, 5, , 6]);
                accessToken = user.accessToken, accessTokenSecret = user.accessTokenSecret;
                if (!tweet) return [3 /*break*/, 2];
                return [4 /*yield*/, postTweet(accessToken, accessTokenSecret, tweet)];
            case 1:
                if (_a.sent()) {
                    console.log(colors.green('✔') + " \u9001\u4FE1\u5B8C\u4E86\n\u30C4\u30A4\u30FC\u30C8: " + tweet);
                }
                return [2 /*return*/];
            case 2: return [4 /*yield*/, inputTweet()];
            case 3:
                input = (_a.sent()).input;
                return [4 /*yield*/, postTweet(accessToken, accessTokenSecret, input)];
            case 4:
                if (_a.sent()) {
                    console.log(colors.green('✔') + " \u9001\u4FE1\u5B8C\u4E86\n\u30C4\u30A4\u30FC\u30C8: " + input);
                }
                return [2 /*return*/];
            case 5:
                err_2 = _a.sent();
                console.error(colors.red('✖') + " " + err_2.message);
                return [3 /*break*/, 6];
            case 6: return [2 /*return*/];
        }
    });
}); };

var viewTimeline = function (data) {
    data.forEach(function (item) {
        console.log(item.name + " " + colors.blue(item.id) + "\n" + item.text + "\n");
    });
};
var getTimeline = function (accessToken, accessTokenSecret, userId) { return __awaiter(void 0, void 0, void 0, function () {
    var data, err_1;
    return __generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                _a.trys.push([0, 3, , 4]);
                return [4 /*yield*/, axios__default['default'].post(middlewareUrl + "/getTimeline", {
                        access_token: accessToken,
                        access_token_secret: accessTokenSecret,
                        user: userId
                    })];
            case 1:
                data = (_a.sent()).data;
                return [4 /*yield*/, viewTimeline(data)];
            case 2:
                _a.sent();
                return [2 /*return*/, true];
            case 3:
                err_1 = _a.sent();
                throw new Error(err_1.message);
            case 4: return [2 /*return*/];
        }
    });
}); };
var timeline = function (user, userId) { return __awaiter(void 0, void 0, void 0, function () {
    var accessToken, accessTokenSecret, err_2;
    return __generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                _a.trys.push([0, 2, , 3]);
                accessToken = user.accessToken, accessTokenSecret = user.accessTokenSecret;
                return [4 /*yield*/, getTimeline(accessToken, accessTokenSecret, userId)];
            case 1:
                _a.sent();
                return [3 /*break*/, 3];
            case 2:
                err_2 = _a.sent();
                console.error(colors.red('✖') + " " + err_2.message);
                return [3 /*break*/, 3];
            case 3: return [2 /*return*/];
        }
    });
}); };

var viewListTimeline = function (data) {
    data.forEach(function (item) {
        console.log(item.name + " " + colors.blue(item.id) + "\n" + item.text + "\n");
    });
};
var getLists = function (accessToken, accessTokenSecret, userid) { return __awaiter(void 0, void 0, void 0, function () {
    var data, err_1;
    return __generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                _a.trys.push([0, 2, , 3]);
                return [4 /*yield*/, axios__default['default'].post(middlewareUrl + "/getList", {
                        access_token: accessToken,
                        access_token_secret: accessTokenSecret,
                        options: {
                            userid: userid
                        }
                    })];
            case 1:
                data = (_a.sent()).data;
                return [2 /*return*/, data];
            case 2:
                err_1 = _a.sent();
                throw new Error(err_1.message);
            case 3: return [2 /*return*/];
        }
    });
}); };
var selectedList = function (lists) { return __awaiter(void 0, void 0, void 0, function () {
    var selected, onCancel, selectedListId;
    return __generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                selected = [];
                return [4 /*yield*/, lists.forEach(function (list) {
                        var shap = {
                            title: list.name + " " + list.description,
                            value: list.id
                        };
                        selected.push(shap);
                    })];
            case 1:
                _a.sent();
                onCancel = function () {
                    throw new Error('選択されませんでした');
                };
                return [4 /*yield*/, prompts__default['default']([
                        {
                            type: 'select',
                            name: 'selectedListId',
                            message: '取得したいリストを選択してください',
                            choices: selected
                        }
                    ], { onCancel: onCancel })];
            case 2:
                selectedListId = (_a.sent()).selectedListId;
                return [2 /*return*/, selectedListId];
        }
    });
}); };
var getList = function (accessToken, accessTokenSecret, options) { return __awaiter(void 0, void 0, void 0, function () {
    var data, err_2;
    return __generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                _a.trys.push([0, 3, , 4]);
                return [4 /*yield*/, axios__default['default'].post(middlewareUrl + "/getList", {
                        access_token: accessToken,
                        access_token_secret: accessTokenSecret,
                        options: options
                    })];
            case 1:
                data = (_a.sent()).data;
                return [4 /*yield*/, viewListTimeline(data)];
            case 2:
                _a.sent();
                return [2 /*return*/];
            case 3:
                err_2 = _a.sent();
                throw new Error(err_2.message);
            case 4: return [2 /*return*/];
        }
    });
}); };
var list = function (user, data) { return __awaiter(void 0, void 0, void 0, function () {
    var accessToken, accessTokenSecret, userid, listid, lists, selectedListId, err_3;
    return __generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                _a.trys.push([0, 6, , 7]);
                accessToken = user.accessToken, accessTokenSecret = user.accessTokenSecret, userid = user.userid;
                if (!Object.keys(data).length) return [3 /*break*/, 2];
                listid = data.listid;
                return [4 /*yield*/, getList(accessToken, accessTokenSecret, {
                        listid: listid
                    })];
            case 1:
                _a.sent();
                return [2 /*return*/];
            case 2: return [4 /*yield*/, getLists(accessToken, accessTokenSecret, userid)];
            case 3:
                lists = _a.sent();
                return [4 /*yield*/, selectedList(lists)];
            case 4:
                selectedListId = _a.sent();
                return [4 /*yield*/, getList(accessToken, accessTokenSecret, {
                        listid: selectedListId
                    })];
            case 5:
                _a.sent();
                return [2 /*return*/];
            case 6:
                err_3 = _a.sent();
                console.error(colors.red('✖') + " " + err_3.message);
                return [3 /*break*/, 7];
            case 7: return [2 /*return*/];
        }
    });
}); };

var setUser$1 = function (db, name) { return __awaiter(void 0, void 0, void 0, function () {
    var completed;
    return __generator(this, function (_a) {
        switch (_a.label) {
            case 0: return [4 /*yield*/, unSetSelecte(db)];
            case 1:
                if (!_a.sent()) return [3 /*break*/, 3];
                return [4 /*yield*/, setSelectedUser(db, name)];
            case 2:
                completed = _a.sent();
                if (completed) {
                    console.log(colors.green('✔') + " " + name + " \u306B\u30A2\u30AB\u30A6\u30F3\u30C8\u3092\u5909\u66F4\u3057\u307E\u3057\u305F");
                    return [2 /*return*/];
                }
                throw new Error('database登録時にエラーがあったようです...もう一度ログインを試してみてください');
            case 3: throw new Error('database設定時にエラーがあったようです...もう一度ログインを試してみてください');
        }
    });
}); };
var createSelectedArray = function (users) {
    var selectArray = [];
    for (var i = 0; i < users.length; i += 1) {
        selectArray.push({
            title: users[i].name,
            value: {
                type: 'user',
                id: users[i]._id,
                name: users[i].name
            }
        });
    }
    return selectArray;
};
var selectedChangeUser = function (selected) { return __awaiter(void 0, void 0, void 0, function () {
    var onCancel, user;
    return __generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                onCancel = function () {
                    throw new Error('選択されませんでした');
                };
                return [4 /*yield*/, prompts__default['default']([
                        {
                            type: 'select',
                            name: 'user',
                            message: '変更したいアカウントを選択してください',
                            choices: selected
                        }
                    ], { onCancel: onCancel })];
            case 1:
                user = (_a.sent()).user;
                return [2 /*return*/, user];
        }
    });
}); };
var selectUser = function (db) { return __awaiter(void 0, void 0, void 0, function () {
    var users, selectedArray, name, err_1;
    return __generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                _a.trys.push([0, 6, , 7]);
                return [4 /*yield*/, getAllUser(db)];
            case 1:
                users = _a.sent();
                return [4 /*yield*/, createSelectedArray(users)];
            case 2:
                selectedArray = _a.sent();
                return [4 /*yield*/, selectedChangeUser(selectedArray)];
            case 3:
                name = (_a.sent()).name;
                if (!name) return [3 /*break*/, 5];
                return [4 /*yield*/, setUser$1(db, name)];
            case 4:
                _a.sent();
                _a.label = 5;
            case 5: return [3 /*break*/, 7];
            case 6:
                err_1 = _a.sent();
                console.error(colors.red('✖') + " " + err_1.message);
                return [3 /*break*/, 7];
            case 7: return [2 /*return*/];
        }
    });
}); };
var user = function (db, name) { return __awaiter(void 0, void 0, void 0, function () {
    var err_2;
    return __generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                _a.trys.push([0, 5, , 6]);
                if (!name) return [3 /*break*/, 4];
                return [4 /*yield*/, checkUserName(db, name)];
            case 1:
                if (!_a.sent()) return [3 /*break*/, 3];
                return [4 /*yield*/, setUser$1(db, name)];
            case 2:
                _a.sent();
                return [2 /*return*/];
            case 3:
                console.log(colors.red('✖') + " \u5165\u529B\u3057\u305F\u540D\u524D\u306F\u898B\u3064\u304B\u308A\u307E\u305B\u3093\u3067\u3057\u305F");
                _a.label = 4;
            case 4:
                selectUser(db);
                return [3 /*break*/, 6];
            case 5:
                err_2 = _a.sent();
                console.error(colors.red('✖') + " " + err_2.message);
                return [3 /*break*/, 6];
            case 6: return [2 /*return*/];
        }
    });
}); };

var getAccessToken = function (oauthToken, oauthTokenSecret, oauthVerifier) { return __awaiter(void 0, void 0, void 0, function () {
    var request, _a, accessToken, accessTokenSecret, id, err_1;
    return __generator(this, function (_b) {
        switch (_b.label) {
            case 0:
                _b.trys.push([0, 2, , 3]);
                return [4 /*yield*/, axios__default['default'].post(middlewareUrl + "/getAccessToken", {
                        oauth_token: oauthToken,
                        oauth_token_secret: oauthTokenSecret,
                        oauth_verifier: oauthVerifier
                    })];
            case 1:
                request = _b.sent();
                _a = request.data, accessToken = _a.accessToken, accessTokenSecret = _a.accessTokenSecret, id = _a.id;
                return [2 /*return*/, { accessToken: accessToken, accessTokenSecret: accessTokenSecret, id: id }];
            case 2:
                err_1 = _b.sent();
                throw new Error(err_1.message);
            case 3: return [2 /*return*/];
        }
    });
}); };
var userInput = function (db) { return __awaiter(void 0, void 0, void 0, function () {
    var onCancel, _a, oauthVerifier, userName;
    return __generator(this, function (_b) {
        switch (_b.label) {
            case 0:
                onCancel = function () {
                    throw new Error('入力内容が確認できませんでした...もう一度最初から入力してください。');
                };
                return [4 /*yield*/, prompts__default['default']([
                        {
                            type: 'text',
                            name: 'oauthVerifier',
                            message: 'ブラウザに表示されたトークンを入力してください',
                            validate: function (value) { return (!value ? '何か入力してください' : true); }
                        },
                        {
                            type: 'text',
                            name: 'userName',
                            message: '表示名を入力してください',
                            validate: function (value) { return __awaiter(void 0, void 0, void 0, function () {
                                return __generator(this, function (_a) {
                                    switch (_a.label) {
                                        case 0:
                                            if (!value) {
                                                return [2 /*return*/, '何か入力してください'];
                                            }
                                            return [4 /*yield*/, checkUserName(db, value)];
                                        case 1:
                                            if (_a.sent()) {
                                                return [2 /*return*/, 'すでにその名前は使われています、他の名前を入力してください'];
                                            }
                                            return [2 /*return*/, true];
                                    }
                                });
                            }); }
                        }
                    ], { onCancel: onCancel })];
            case 1:
                _a = _b.sent(), oauthVerifier = _a.oauthVerifier, userName = _a.userName;
                return [2 /*return*/, { oauthVerifier: oauthVerifier, userName: userName }];
        }
    });
}); };
var getOauthToken = function () { return __awaiter(void 0, void 0, void 0, function () {
    var request, _a, oauthToken, oauthTokenSecret, err_2;
    return __generator(this, function (_b) {
        switch (_b.label) {
            case 0:
                _b.trys.push([0, 2, , 3]);
                return [4 /*yield*/, axios__default['default'].post(middlewareUrl + "/getOauthToken")];
            case 1:
                request = _b.sent();
                _a = request.data, oauthToken = _a.oauthToken, oauthTokenSecret = _a.oauthTokenSecret;
                return [2 /*return*/, { oauthToken: oauthToken, oauthTokenSecret: oauthTokenSecret }];
            case 2:
                err_2 = _b.sent();
                throw new Error(err_2.message);
            case 3: return [2 /*return*/];
        }
    });
}); };
var Login = function (db) { return __awaiter(void 0, void 0, void 0, function () {
    var spinner, _a, oauthToken, oauthTokenSecret, _b, oauthVerifier, userName, _c, accessToken, accessTokenSecret, id, err_3;
    return __generator(this, function (_d) {
        switch (_d.label) {
            case 0:
                _d.trys.push([0, 12, , 13]);
                spinner = ora__default['default']('通信中...').start();
                return [4 /*yield*/, getOauthToken()];
            case 1:
                _a = _d.sent(), oauthToken = _a.oauthToken, oauthTokenSecret = _a.oauthTokenSecret;
                spinner.succeed('準備完了!');
                if (!(oauthToken && oauthTokenSecret)) return [3 /*break*/, 3];
                return [4 /*yield*/, open__default['default'](callbackUrl + "/authenticate?oauth_token=" + oauthToken)];
            case 2:
                _d.sent();
                return [3 /*break*/, 4];
            case 3: throw new Error('Twitter APIに問題があるようです。時間開けてもう一度お試しください');
            case 4: return [4 /*yield*/, userInput(db)];
            case 5:
                _b = _d.sent(), oauthVerifier = _b.oauthVerifier, userName = _b.userName;
                return [4 /*yield*/, getAccessToken(oauthToken, oauthTokenSecret, oauthVerifier)];
            case 6:
                _c = _d.sent(), accessToken = _c.accessToken, accessTokenSecret = _c.accessTokenSecret, id = _c.id;
                if (!(accessToken && accessTokenSecret && id)) return [3 /*break*/, 10];
                return [4 /*yield*/, unSetSelecte(db)];
            case 7:
                if (!_d.sent()) return [3 /*break*/, 9];
                return [4 /*yield*/, setUser(db, accessToken, accessTokenSecret, userName, id)];
            case 8:
                if (_d.sent()) {
                    console.log(colors.green('✔') + " \u3088\u3046\u3053\u305D\u3001" + userName + "!");
                    return [2 /*return*/];
                }
                _d.label = 9;
            case 9: throw new Error('databaseに問題があります');
            case 10: throw new Error('入力されたTokenに問題があります');
            case 11: return [3 /*break*/, 13];
            case 12:
                err_3 = _d.sent();
                console.error(colors.red('✖') + " " + err_3.message);
                return [3 /*break*/, 13];
            case 13: return [2 /*return*/];
        }
    });
}); };

var deleteSelectedUser = function (db, id, userName) { return __awaiter(void 0, void 0, void 0, function () {
    var result, checkUser, name;
    return __generator(this, function (_a) {
        switch (_a.label) {
            case 0: return [4 /*yield*/, deleteUser(db, id)];
            case 1:
                result = _a.sent();
                if (!result) return [3 /*break*/, 6];
                console.log(colors.green('✔') + " " + userName + "\u304B\u3089\u30ED\u30B0\u30A2\u30A6\u30C8\u3057\u307E\u3057\u305F");
                return [4 /*yield*/, selectedUser(db)];
            case 2:
                checkUser = _a.sent();
                if (!!Object.keys(checkUser).length) return [3 /*break*/, 5];
                return [4 /*yield*/, getRandomUser(db)];
            case 3:
                name = (_a.sent()).name;
                return [4 /*yield*/, setSelectedUser(db, name)];
            case 4:
                if (_a.sent()) {
                    console.log(colors.green('✔') + " \u73FE\u5728\u306E\u30E6\u30FC\u30B6\u30FC\u306F" + name + "\u3067\u3059");
                    return [2 /*return*/, true];
                }
                throw new Error('database設定時にエラーがあったようです...もう一度ログインを試してみてください');
            case 5: return [2 /*return*/, true];
            case 6: throw new Error('database削除時にエラーがあったようです...もう一度ログインを試してみてください');
        }
    });
}); };
var selectedDeleteUser = function (selected) { return __awaiter(void 0, void 0, void 0, function () {
    var onCancel, user;
    return __generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                onCancel = function () {
                    throw new Error('選択されませんでした');
                };
                return [4 /*yield*/, prompts__default['default']([
                        {
                            type: 'select',
                            name: 'user',
                            message: '削除したいアカウントを選択してください',
                            choices: selected
                        }
                    ], { onCancel: onCancel })];
            case 1:
                user = (_a.sent()).user;
                return [2 /*return*/, user];
        }
    });
}); };
var createSelectedArray$1 = function (users) {
    var selectArray = [
        {
            title: 'all',
            value: { type: 'all' }
        }
    ];
    for (var i = 0; i < users.length; i += 1) {
        selectArray.push({
            title: users[i].name,
            value: {
                type: 'user',
                id: users[i]._id,
                name: users[i].name
            }
        });
    }
    return selectArray;
};
var Logout = function (db, path) { return __awaiter(void 0, void 0, void 0, function () {
    var users, selectedArray, _a, type, id, name, err_1;
    return __generator(this, function (_b) {
        switch (_b.label) {
            case 0:
                _b.trys.push([0, 6, , 7]);
                return [4 /*yield*/, getAllUser(db)];
            case 1:
                users = _b.sent();
                return [4 /*yield*/, createSelectedArray$1(users)];
            case 2:
                selectedArray = _b.sent();
                return [4 /*yield*/, selectedDeleteUser(selectedArray)];
            case 3:
                _a = _b.sent(), type = _a.type, id = _a.id, name = _a.name;
                if (type === 'all') {
                    fs__default['default'].unlinkSync(path);
                    console.log(colors.green('✔') + " \u5168\u3066\u306E\u30A2\u30AB\u30A6\u30F3\u30C8\u3092\u524A\u9664\u3057\u307E\u3057\u305F");
                    return [2 /*return*/];
                }
                if (!type) return [3 /*break*/, 5];
                return [4 /*yield*/, deleteSelectedUser(db, id, name)];
            case 4:
                _b.sent();
                _b.label = 5;
            case 5: return [2 /*return*/];
            case 6:
                err_1 = _b.sent();
                console.error(colors.red('✖') + " " + err_1.message);
                return [3 /*break*/, 7];
            case 7: return [2 /*return*/];
        }
    });
}); };

var path = __dirname + "/configs/database";
var main = function () { return __awaiter(void 0, void 0, void 0, function () {
    var db, loginUser, userId, NO_COMMAND_SPECIFIED;
    return __generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                updateNotifier__default['default']({
                    packageName: pjson__default['default'].name,
                    packageVersion: pjson__default['default'].version
                }).notify();
                return [4 /*yield*/, new Nedb__default['default']({
                        filename: path
                    })];
            case 1:
                db = _a.sent();
                return [4 /*yield*/, db.loadDatabase()];
            case 2:
                _a.sent();
                return [4 /*yield*/, selectedUser(db)];
            case 3:
                loginUser = _a.sent();
                program__default['default']
                    .version(pjson__default['default'].version, '-v, --version', 'バージョンを確認')
                    .option('-l, --login', 'ログイン')
                    .option('-lo, --logout', 'ログアウト')
                    .option('-t, --tweet [tweet]', 'ツイート')
                    .option('-tl, --timeline [user]', 'タイムラインを取得')
                    .option('-li, --list [user/list]', 'リストを取得')
                    .option('-c, --change [user]', 'アカウント切替')
                    .parse(process.argv);
                if (!!Object.keys(loginUser).length) return [3 /*break*/, 5];
                return [4 /*yield*/, Login(db)];
            case 4:
                _a.sent();
                return [2 /*return*/];
            case 5:
                if (program__default['default'].login)
                    Login(db);
                if (program__default['default'].logout)
                    Logout(db, path);
                if (program__default['default'].tweet) {
                    if (typeof program__default['default'].tweet === 'string') {
                        Tweet(loginUser, program__default['default'].tweet);
                        return [2 /*return*/];
                    }
                    Tweet(loginUser, '');
                    return [2 /*return*/];
                }
                if (program__default['default'].timeline) {
                    if (typeof program__default['default'].timeline === 'string') {
                        if (program__default['default'].timeline.charAt(0) === '@') {
                            userId = program__default['default'].timeline.slice(1);
                            timeline(loginUser, userId);
                            return [2 /*return*/];
                        }
                        timeline(loginUser, program__default['default'].timeline);
                        return [2 /*return*/];
                    }
                    timeline(loginUser, '');
                    return [2 /*return*/];
                }
                if (program__default['default'].list) {
                    if (typeof program__default['default'].list === 'string') {
                        list(loginUser, {
                            listid: program__default['default'].list
                        });
                        return [2 /*return*/];
                    }
                    list(loginUser, {});
                    return [2 /*return*/];
                }
                if (program__default['default'].change) {
                    if (typeof program__default['default'].change === 'string') {
                        user(db, program__default['default'].change);
                        return [2 /*return*/];
                    }
                    user(db, '');
                    return [2 /*return*/];
                }
                NO_COMMAND_SPECIFIED = process.argv.length <= 2;
                if (NO_COMMAND_SPECIFIED) {
                    if (loginUser.name) {
                        console.log(colors.green('✔') + " " + loginUser.name + " \u3067\u30ED\u30B0\u30A4\u30F3\u3057\u3066\u3044\u307E\u3059");
                        return [2 /*return*/];
                    }
                    console.log('ログインしていません');
                }
                return [2 /*return*/];
        }
    });
}); };

var index = main();

module.exports = index;
